@@ -872,6 +872,11 @@ If you have questions concerning this license or the applicable additional terms
 #define ID_SELECTION_VIEW_WIREFRAMEOFF           30403
 #define ID_SELECTION_VIEW_VISIBLEON              30404
 #define ID_SELECTION_VIEW_VISIBLEOFF             30405
+/* Begin SS2 changes */
+#define ID_SET_VIEW_TOP                          30406
+#define ID_SET_VIEW_SIDE                         30407
+#define ID_SET_VIEW_FRONT                        30408
+/* End changes */
 #define ID_ENTITY_START                          40000
 #define ID_ENTITY_END                            45000
 
