@@ -51,6 +51,8 @@ If you have questions concerning this license or the applicable additional terms
 #include <sys/stat.h>
 #endif
 
+#include "tools/edit_public.h"
+
 #include <SDL_main.h>
 
 idCVar Win32Vars_t::win_outputDebugString( "win_outputDebugString", "0", CVAR_SYSTEM | CVAR_BOOL, "" );
@@ -59,6 +61,61 @@ idCVar Win32Vars_t::win_viewlog( "win_viewlog", "0", CVAR_SYSTEM | CVAR_INTEGER,
 
 Win32Vars_t	win32;
 
+#ifdef ID_ALLOW_TOOLS
+/* These are required for tools (DG: taken from SteelStorm2) */
+
+static HMODULE hOpenGL_DLL;
+
+typedef BOOL(WINAPI * PWGLSWAPBUFFERS)(HDC);
+
+PWGLSWAPBUFFERS			qwglSwapBuffers;
+
+typedef BOOL(WINAPI * PWGLCOPYCONTEXT)(HGLRC, HGLRC, UINT);
+typedef HGLRC(WINAPI * PWGLCREATECONTEXT)(HDC);
+typedef HGLRC(WINAPI * PWGLCREATELAYERCONTEXT)(HDC, int);
+typedef BOOL(WINAPI * PWGLDELETECONTEXT)(HGLRC);
+typedef HGLRC(WINAPI * PWGLGETCURRENTCONTEXT)(VOID);
+typedef HDC(WINAPI * PWGLGETCURRENTDC)(VOID);
+typedef PROC(WINAPI * PWGLGETPROCADDRESS)(LPCSTR);
+typedef BOOL(WINAPI * PWGLMAKECURRENT)(HDC, HGLRC);
+typedef BOOL(WINAPI * PWGLSHARELISTS)(HGLRC, HGLRC);
+typedef BOOL(WINAPI * PWGLUSEFONTBITMAPS)(HDC, DWORD, DWORD, DWORD);
+
+
+PWGLCOPYCONTEXT			qwglCopyContext;
+PWGLCREATECONTEXT		qwglCreateContext;
+PWGLCREATELAYERCONTEXT	qwglCreateLayerContext;
+PWGLDELETECONTEXT		qwglDeleteContext;
+PWGLGETCURRENTCONTEXT	qwglGetCurrentContext;
+PWGLGETCURRENTDC		qwglGetCurrentDC;
+PWGLGETPROCADDRESS		qwglGetProcAddress;
+PWGLMAKECURRENT			qwglMakeCurrent;
+PWGLSHARELISTS			qwglShareLists;
+PWGLUSEFONTBITMAPS		qwglUseFontBitmaps;
+
+typedef BOOL(WINAPI * PWGLUSEFONTOUTLINES)(HDC, DWORD, DWORD, DWORD, FLOAT,
+	FLOAT, int, LPGLYPHMETRICSFLOAT);
+typedef BOOL(WINAPI * PWGLDESCRIBELAYERPLANE)(HDC, int, int, UINT,
+	LPLAYERPLANEDESCRIPTOR);
+typedef int  (WINAPI * PWGLSETLAYERPALETTEENTRIES)(HDC, int, int, int,
+	CONST COLORREF *);
+typedef int  (WINAPI * PWGLGETLAYERPALETTEENTRIES)(HDC, int, int, int,
+	COLORREF *);
+typedef BOOL(WINAPI * PWGLREALIZELAYERPALETTE)(HDC, int, BOOL);
+typedef BOOL(WINAPI * PWGLSWAPLAYERBUFFERS)(HDC, UINT);
+
+PWGLUSEFONTOUTLINES			qwglUseFontOutlines;
+PWGLDESCRIBELAYERPLANE		qwglDescribeLayerPlane;
+PWGLSETLAYERPALETTEENTRIES	qwglSetLayerPaletteEntries;
+PWGLGETLAYERPALETTEENTRIES	qwglGetLayerPaletteEntries;
+PWGLREALIZELAYERPALETTE		qwglRealizeLayerPalette;
+PWGLSWAPLAYERBUFFERS		qwglSwapLayerBuffers;
+
+#endif /* End stuff required for tools */
+
+static bool hadError = false;
+static char errorText[4096];
+
 /*
 =============
 Sys_Error
@@ -71,11 +128,28 @@ void Sys_Error( const char *error, ... ) {
 	char		text[4096];
 	MSG        msg;
 
+	if ( !Sys_IsMainThread() ) {
+		// to avoid deadlocks we mustn't call Conbuf_AppendText() etc if not in main thread!
+		va_start(argptr, error);
+		vsprintf(errorText, error, argptr);
+		va_end(argptr);
+
+		printf("%s", errorText);
+		OutputDebugString( errorText );
+
+		hadError = true;
+		return;
+	}
+
 	va_start( argptr, error );
 	vsprintf( text, error, argptr );
 	va_end( argptr);
 
-	printf("%s", text);
+	if ( !hadError ) {
+		// if we had an error in another thread, printf() and OutputDebugString() has already been called for this
+		printf( "%s", text );
+		OutputDebugString( text );
+	}
 
 	Conbuf_AppendText( text );
 	Conbuf_AppendText( "\n" );
@@ -109,6 +183,14 @@ Sys_Quit
 ==============
 */
 void Sys_Quit( void ) {
+#ifdef ID_ALLOW_TOOLS
+	// Free OpenGL DLL.
+	if (hOpenGL_DLL)
+	{
+		FreeLibrary(hOpenGL_DLL);
+	}
+#endif
+
 	timeEndPeriod( 1 );
 	Sys_ShutdownInput();
 	Sys_DestroyConsole();
@@ -121,13 +203,22 @@ void Sys_Quit( void ) {
 Sys_Printf
 ==============
 */
-#define MAXPRINTMSG 4096
+
+enum {
+	MAXPRINTMSG = 4096,
+	MAXNUMBUFFEREDLINES = 16
+};
+
+static char bufferedPrintfLines[MAXNUMBUFFEREDLINES][MAXPRINTMSG];
+static int curNumBufferedPrintfLines = 0;
+static CRITICAL_SECTION printfCritSect;
+
 void Sys_Printf( const char *fmt, ... ) {
 	char		msg[MAXPRINTMSG];
 
 	va_list argptr;
 	va_start(argptr, fmt);
-	idStr::vsnPrintf( msg, MAXPRINTMSG-1, fmt, argptr );
+	int len = idStr::vsnPrintf( msg, MAXPRINTMSG-1, fmt, argptr );
 	va_end(argptr);
 	msg[sizeof(msg)-1] = '\0';
 
@@ -137,7 +228,18 @@ void Sys_Printf( const char *fmt, ... ) {
 		OutputDebugString( msg );
 	}
 	if ( win32.win_outputEditString.GetBool() ) {
-		Conbuf_AppendText( msg );
+		if ( Sys_IsMainThread() ) {
+			Conbuf_AppendText( msg );
+		} else {
+			EnterCriticalSection( &printfCritSect );
+			int idx = curNumBufferedPrintfLines++;
+			if ( idx < MAXNUMBUFFEREDLINES ) {
+				if ( len >= MAXPRINTMSG )
+					len = MAXPRINTMSG - 1;
+				memcpy( bufferedPrintfLines[idx], msg, len + 1 );
+			}
+			LeaveCriticalSection( &printfCritSect );
+		}
 	}
 }
 
@@ -275,7 +377,7 @@ static int WPath2A(char *dst, size_t size, const WCHAR *src) {
 /*
 ==============
 Returns "My Documents"/My Games/dhewm3 directory (or equivalent - "CSIDL_PERSONAL").
-To be used with Sys_DefaultSavePath(), so savegames, screenshots etc will be
+To be used with Sys_GetPath(PATH_SAVE), so savegames, screenshots etc will be
 saved to the users files instead of systemwide.
 
 Based on (with kind permission) Yamagi Quake II's Sys_GetHomeDir()
@@ -283,21 +385,23 @@ Based on (with kind permission) Yamagi Quake II's Sys_GetHomeDir()
 Returns the number of characters written to dst
 ==============
  */
-static int GetHomeDir(char *dst, size_t size)
-{
-	int len;
-	WCHAR profile[MAX_OSPATH];
+extern "C" { // DG: I need this in SDL_win32_main.c
+	int Win_GetHomeDir(char *dst, size_t size)
+	{
+		int len;
+		WCHAR profile[MAX_OSPATH];
 
-	/* Get the path to "My Documents" directory */
-	SHGetFolderPathW(NULL, CSIDL_PERSONAL, NULL, 0, profile);
+		/* Get the path to "My Documents" directory */
+		SHGetFolderPathW(NULL, CSIDL_PERSONAL, NULL, 0, profile);
 
-	len = WPath2A(dst, size, profile);
-	if (len == 0)
-		return 0;
+		len = WPath2A(dst, size, profile);
+		if (len == 0)
+			return 0;
 
-	idStr::Append(dst, size, "/My Games/dhewm3");
+		idStr::Append(dst, size, "/My Games/dhewm3");
 
-	return len;
+		return len;
+	}
 }
 
 static int GetRegistryPath(char *dst, size_t size, const WCHAR *subkey, const WCHAR *name) {
@@ -340,12 +444,22 @@ bool Sys_GetPath(sysPath_t type, idStr &path) {
 
 			s = path;
 			s.AppendPath(BASE_GAMEDIR);
-			if (_stat(s.c_str(), &st) != -1 && st.st_mode & _S_IFDIR)
+			if (_stat(s.c_str(), &st) != -1 && (st.st_mode & _S_IFDIR)) {
+				common->Warning("using path of executable: %s", path.c_str());
 				return true;
+			} else {
+				s = path + "/demo/demo00.pk4";
+				if (_stat(s.c_str(), &st) != -1 && (st.st_mode & _S_IFREG)) {
+					common->Warning("using path of executable (seems to contain demo game data): %s ", path.c_str());
+					return true;
+				}
+			}
 
 			common->Warning("base path '%s' does not exist", s.c_str());
 		}
 
+		// Note: apparently there is no registry entry for the Doom 3 Demo
+
 		// fallback to vanilla doom3 cd install
 		if (GetRegistryPath(buf, sizeof(buf), L"SOFTWARE\\id\\Doom 3", L"InstallPath") > 0) {
 			path = buf;
@@ -361,13 +475,13 @@ bool Sys_GetPath(sysPath_t type, idStr &path) {
 				return true;
 		}
 
-		common->Warning("vanilla doom3 path not found");
+		common->Warning("vanilla doom3 path not found either");
 
 		return false;
 
 	case PATH_CONFIG:
 	case PATH_SAVE:
-		if (GetHomeDir(buf, sizeof(buf)) < 1) {
+		if (Win_GetHomeDir(buf, sizeof(buf)) < 1) {
 			Sys_Error("ERROR: Couldn't get dir to home path");
 			return false;
 		}
@@ -456,6 +570,10 @@ char *Sys_GetClipboardData( void ) {
 	return data;
 }
 
+void Sys_FreeClipboardData( char* data ) {
+	Mem_Free( data );
+}
+
 /*
 ================
 Sys_SetClipboardData
@@ -518,6 +636,33 @@ uintptr_t Sys_DLL_Load( const char *dllName ) {
 			Sys_DLL_Unload( (uintptr_t)libHandle );
 			return 0;
 		}
+	} else {
+		DWORD e = GetLastError();
+		LPVOID msgBuf = NULL;
+
+		FormatMessage(
+			FORMAT_MESSAGE_ALLOCATE_BUFFER |
+			FORMAT_MESSAGE_FROM_SYSTEM |
+			FORMAT_MESSAGE_IGNORE_INSERTS,
+			NULL,
+			e,
+			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+			(LPTSTR)&msgBuf,
+			0, NULL);
+
+		idStr errorStr = va( "[%i (0x%X)]\t%s", e, e, msgBuf );
+
+		// common, skipped.
+		if ( e == 0x7E ) // [126 (0x7E)] The specified module could not be found.
+			errorStr = "";
+		// probably going to be common. Lets try to be less cryptic.
+		else if ( e == 0xC1 ) // [193 (0xC1)] is not a valid Win32 application.
+			errorStr = va( "[%i (0x%X)]\t%s", e, e, "probably the DLL is of the wrong architecture, like x64 instead of x86" );
+
+		if ( errorStr.Length() )
+			common->Warning( "LoadLibrary(%s) Failed ! %s", dllName, errorStr.c_str() );
+
+		::LocalFree( msgBuf );
 	}
 	return (uintptr_t)libHandle;
 }
@@ -528,7 +673,30 @@ Sys_DLL_GetProcAddress
 =====================
 */
 void *Sys_DLL_GetProcAddress( uintptr_t dllHandle, const char *procName ) {
-	return (void *)GetProcAddress( (HINSTANCE)dllHandle, procName );
+	void * adr = (void*)GetProcAddress((HINSTANCE)dllHandle, procName);
+	if (!adr)
+	{
+		DWORD e = GetLastError();
+		LPVOID msgBuf = NULL;
+
+		FormatMessage(
+			FORMAT_MESSAGE_ALLOCATE_BUFFER |
+			FORMAT_MESSAGE_FROM_SYSTEM |
+			FORMAT_MESSAGE_IGNORE_INSERTS,
+			NULL,
+			e,
+			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+			(LPTSTR)&msgBuf,
+			0, NULL);
+
+		idStr errorStr = va("[%i (0x%X)]\t%s", e, e, msgBuf);
+
+		if (errorStr.Length())
+			common->Warning("GetProcAddress( %i %s) Failed ! %s", dllHandle, procName, errorStr.c_str());
+
+		::LocalFree(msgBuf);
+	}
+	return adr;
 }
 
 /*
@@ -580,6 +748,11 @@ void Sys_Init( void ) {
 #if 0
 	cmdSystem->AddCommand( "setAsyncSound", Sys_SetAsyncSound_f, CMD_FL_SYSTEM, "set the async sound option" );
 #endif
+	{
+		idStr savepath;
+		Sys_GetPath( PATH_SAVE, savepath );
+		common->Printf( "Logging console output to %s/dhewm3log.txt\n", savepath.c_str() );
+	}
 
 	//
 	// Windows version
@@ -605,6 +778,26 @@ Sys_Shutdown
 ================
 */
 void Sys_Shutdown( void ) {
+#ifdef ID_ALLOW_TOOLS
+	qwglCopyContext = NULL;
+	qwglCreateContext = NULL;
+	qwglCreateLayerContext = NULL;
+	qwglDeleteContext = NULL;
+	qwglDescribeLayerPlane = NULL;
+	qwglGetCurrentContext = NULL;
+	qwglGetCurrentDC = NULL;
+	qwglGetLayerPaletteEntries = NULL;
+	qwglGetProcAddress = NULL;
+	qwglMakeCurrent = NULL;
+	qwglRealizeLayerPalette = NULL;
+	qwglSetLayerPaletteEntries = NULL;
+	qwglShareLists = NULL;
+	qwglSwapLayerBuffers = NULL;
+	qwglUseFontBitmaps = NULL;
+	qwglUseFontOutlines = NULL;
+	qwglSwapBuffers = NULL;
+#endif // ID_ALLOW_TOOLS
+
 	CoUninitialize();
 }
 
@@ -626,8 +819,75 @@ void Win_Frame( void ) {
 		}
 		win32.win_viewlog.ClearModified();
 	}
+
+	if ( curNumBufferedPrintfLines > 0 ) {
+		// if Sys_Printf() had been called in another thread, add those lines to the windows console now
+		EnterCriticalSection( &printfCritSect );
+		int n = Min( curNumBufferedPrintfLines, (int)MAXNUMBUFFEREDLINES );
+		for ( int i = 0; i < n; ++i ) {
+			Conbuf_AppendText( bufferedPrintfLines[i] );
+		}
+		curNumBufferedPrintfLines = 0;
+		LeaveCriticalSection( &printfCritSect );
+	}
+
+	if ( hadError ) {
+		// if Sys_Error() had been called in another thread, handle it now
+		Sys_Error( "%s", errorText );
+	}
 }
 
+// the MFC tools use Win_GetWindowScalingFactor() for High-DPI support
+#ifdef ID_ALLOW_TOOLS
+
+typedef enum D3_MONITOR_DPI_TYPE {
+	D3_MDT_EFFECTIVE_DPI = 0,
+	D3_MDT_ANGULAR_DPI = 1,
+	D3_MDT_RAW_DPI = 2,
+	D3_MDT_DEFAULT = D3_MDT_EFFECTIVE_DPI
+} D3_MONITOR_DPI_TYPE;
+
+// https://docs.microsoft.com/en-us/windows/win32/api/shellscalingapi/nf-shellscalingapi-getdpiformonitor
+// GetDpiForMonitor() - Win8.1+, shellscalingapi.h, Shcore.dll
+static HRESULT (STDAPICALLTYPE *D3_GetDpiForMonitor)(HMONITOR hmonitor, D3_MONITOR_DPI_TYPE dpiType, UINT *dpiX, UINT *dpiY) = NULL;
+
+// https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-getdpiforwindow
+// GetDpiForWindow() - Win10 1607+, winuser.h/Windows.h, User32.dll
+static UINT(WINAPI *D3_GetDpiForWindow)(HWND hwnd) = NULL;
+
+float Win_GetWindowScalingFactor(HWND window)
+{
+	// the best way - supported by Win10 1607 and newer
+	if ( D3_GetDpiForWindow != NULL ) {
+		UINT dpi = D3_GetDpiForWindow(window);
+		return static_cast<float>(dpi) / 96.0f;
+	}
+
+	// probably second best, supported by Win8.1 and newer
+	if ( D3_GetDpiForMonitor != NULL ) {
+		HMONITOR monitor = MonitorFromWindow(window, MONITOR_DEFAULTTOPRIMARY);
+		UINT dpiX = 96, dpiY;
+		D3_GetDpiForMonitor(monitor, D3_MDT_EFFECTIVE_DPI, &dpiX, &dpiY);
+		return static_cast<float>(dpiX) / 96.0f;
+	}
+
+	// on older versions of windows, DPI was system-wide (not per monitor)
+	// and changing DPI required logging out and in again (AFAIK), so we only need to get it once
+	static float scaling_factor = -1.0f;
+	if ( scaling_factor == -1.0f ) {
+		HDC hdc = GetDC(window);
+		if (hdc == NULL) {
+			return 1.0f;
+		}
+		// "Number of pixels per logical inch along the screen width. In a system with multiple display monitors, this value is the same for all monitors."
+		int ppi = GetDeviceCaps(hdc, LOGPIXELSX);
+		scaling_factor = static_cast<float>(ppi) / 96.0f;
+	}
+	return scaling_factor;
+}
+
+#endif // ID_ALLOW_TOOLS
+
 // code that tells windows we're High DPI aware so it doesn't scale our windows
 // taken from Yamagi Quake II
 
@@ -635,7 +895,7 @@ typedef enum D3_PROCESS_DPI_AWARENESS {
 	D3_PROCESS_DPI_UNAWARE = 0,
 	D3_PROCESS_SYSTEM_DPI_AWARE = 1,
 	D3_PROCESS_PER_MONITOR_DPI_AWARE = 2
-} YQ2_PROCESS_DPI_AWARENESS;
+} D3_PROCESS_DPI_AWARENESS;
 
 static void setHighDPIMode(void)
 {
@@ -645,7 +905,6 @@ static void setHighDPIMode(void)
 	/* Win8.1 and later */
 	HRESULT(WINAPI *SetProcessDpiAwareness)(D3_PROCESS_DPI_AWARENESS dpiAwareness) = NULL;
 
-
 	HINSTANCE userDLL = LoadLibrary("USER32.DLL");
 
 	if (userDLL)
@@ -653,32 +912,115 @@ static void setHighDPIMode(void)
 		SetProcessDPIAware = (BOOL(WINAPI *)(void)) GetProcAddress(userDLL, "SetProcessDPIAware");
 	}
 
-
 	HINSTANCE shcoreDLL = LoadLibrary("SHCORE.DLL");
 
 	if (shcoreDLL)
 	{
-		SetProcessDpiAwareness = (HRESULT(WINAPI *)(YQ2_PROCESS_DPI_AWARENESS))
+		SetProcessDpiAwareness = (HRESULT(WINAPI *)(D3_PROCESS_DPI_AWARENESS))
 									GetProcAddress(shcoreDLL, "SetProcessDpiAwareness");
 	}
 
-
 	if (SetProcessDpiAwareness) {
 		SetProcessDpiAwareness(D3_PROCESS_PER_MONITOR_DPI_AWARE);
 	}
 	else if (SetProcessDPIAware) {
 		SetProcessDPIAware();
 	}
+
+#ifdef ID_ALLOW_TOOLS // also init function pointers for Win_GetWindowScalingFactor() here
+	if (userDLL) {
+		D3_GetDpiForWindow = (UINT(WINAPI *)(HWND))GetProcAddress(userDLL, "GetDpiForWindow");
+	}
+	if (shcoreDLL) {
+		D3_GetDpiForMonitor = (HRESULT (STDAPICALLTYPE *)(HMONITOR, D3_MONITOR_DPI_TYPE, UINT *, UINT *))
+		                          GetProcAddress(shcoreDLL, "GetDpiForMonitor");
+	}
+#endif // ID_ALLOW_TOOLS
 }
 
+#ifdef ID_ALLOW_TOOLS
+static void loadWGLpointers() {
+	if (hOpenGL_DLL == NULL)
+	{
+		// Load OpenGL DLL.
+		hOpenGL_DLL = LoadLibrary("opengl32.dll");
+		if (hOpenGL_DLL == NULL) {
+			Sys_Error(GAME_NAME " Cannot Load opengl32.dll - Disabling TOOLS");
+			return;
+		}
+	}
+	// opengl32.dll found... grab the addresses.
+
+	qwglGetProcAddress = (PWGLGETPROCADDRESS)GetProcAddress(hOpenGL_DLL, "wglGetProcAddress");
+
+	// Context controls
+	qwglCopyContext = (PWGLCOPYCONTEXT)GetProcAddress(hOpenGL_DLL, "wglCopyContext");
+	qwglCreateContext = (PWGLCREATECONTEXT)GetProcAddress(hOpenGL_DLL, "wglCreateContext");
+	qwglCreateLayerContext = (PWGLCREATELAYERCONTEXT)GetProcAddress(hOpenGL_DLL, "wglCreateLayerContext");
+	qwglDeleteContext = (PWGLDELETECONTEXT)GetProcAddress(hOpenGL_DLL, "wglDeleteContext");
+	qwglGetCurrentContext = (PWGLGETCURRENTCONTEXT)GetProcAddress(hOpenGL_DLL, "wglGetCurrentContext");
+	qwglGetCurrentDC = (PWGLGETCURRENTDC)GetProcAddress(hOpenGL_DLL, "wglGetCurrentDC");
+	qwglMakeCurrent = (PWGLMAKECURRENT)GetProcAddress(hOpenGL_DLL, "wglMakeCurrent");
+	qwglShareLists = (PWGLSHARELISTS)GetProcAddress(hOpenGL_DLL, "wglShareLists");
+
+	// Fonts
+	qwglUseFontBitmaps = (PWGLUSEFONTBITMAPS)GetProcAddress(hOpenGL_DLL, "wglUseFontBitmapsA");
+	qwglUseFontOutlines = (PWGLUSEFONTOUTLINES)GetProcAddress(hOpenGL_DLL, "wglUseFontOutlinesA");
+
+	// Layers.
+	qwglDescribeLayerPlane = (PWGLDESCRIBELAYERPLANE)GetProcAddress(hOpenGL_DLL, "wglDescribeLayerPlane");
+	qwglSwapLayerBuffers = (PWGLSWAPLAYERBUFFERS)GetProcAddress(hOpenGL_DLL, "wglSwapLayerBuffers");
+
+	// Palette controls
+	qwglGetLayerPaletteEntries = (PWGLGETLAYERPALETTEENTRIES)GetProcAddress(hOpenGL_DLL, "wglGetLayerPaletteEntries");
+	qwglRealizeLayerPalette = (PWGLREALIZELAYERPALETTE)GetProcAddress(hOpenGL_DLL, "wglRealizeLayerPalette");
+	qwglSetLayerPaletteEntries = (PWGLSETLAYERPALETTEENTRIES)GetProcAddress(hOpenGL_DLL, "wglSetLayerPaletteEntries");
+
+
+	// These by default exist in windows
+	qwglSwapBuffers = SwapBuffers;
+}
+
+// calls wglChoosePixelFormatARB() or ChoosePixelFormat() matching the main window from SDL
+int Win_ChoosePixelFormat(HDC hdc)
+{
+	if (win32.wglChoosePixelFormatARB != NULL && win32.piAttribIList != NULL) {
+		int formats[4];
+		UINT numFormats = 0;
+		if (win32.wglChoosePixelFormatARB(hdc, win32.piAttribIList, NULL, 4, formats, &numFormats) && numFormats > 0) {
+			return formats[0];
+		}
+		static bool haveWarned = false;
+		if(!haveWarned) {
+			common->Warning("wglChoosePixelFormatARB() failed, falling back to ChoosePixelFormat()!\n");
+			haveWarned = true;
+		}
+	}
+	// fallback to normal ChoosePixelFormats() - doesn't support MSAA!
+	return ChoosePixelFormat(hdc, &win32.pfd);
+}
+#endif
+
 /*
 ==================
 WinMain
 ==================
 */
 int main(int argc, char *argv[]) {
+	// SDL_win32_main.c creates the dhewm3log.txt and redirects stdout into it
+	// so here we can log its (approx.) creation time before anything else is logged:
+	{
+		time_t tt = time(NULL);
+		const struct tm* tms = localtime(&tt);
+		char timeStr[64] = {};
+		strftime(timeStr, sizeof(timeStr), "%F %H:%M:%S", tms);
+		printf("Opened this log at %s\n", timeStr);
+	}
+
 	const HCURSOR hcurSave = ::SetCursor( LoadCursor( 0, IDC_WAIT ) );
 
+	InitializeCriticalSection( &printfCritSect );
+
 #ifdef ID_DEDICATED
 	MSG msg;
 #else
@@ -701,6 +1043,10 @@ int main(int argc, char *argv[]) {
 	_CrtSetDbgFlag( 0 );
 #endif
 
+#ifdef ID_ALLOW_TOOLS
+	loadWGLpointers();
+#endif
+
 	if ( argc > 1 ) {
 		common->Init( argc-1, &argv[1] );
 	} else {
@@ -723,7 +1069,10 @@ int main(int argc, char *argv[]) {
 
 	// Launch the script debugger
 	if ( strstr( GetCommandLine(), "+debugger" ) ) {
-		// DebuggerClientInit( lpCmdLine );
+
+#ifdef ID_ALLOW_TOOLS
+		DebuggerClientInit(GetCommandLine());
+#endif
 		return 0;
 	}
 
@@ -844,6 +1193,7 @@ void idSysLocal::StartProcess( const char *exePath, bool doexit ) {
 	si.cb = sizeof(si);
 
 	strncpy( szPathOrig, exePath, _MAX_PATH );
+	szPathOrig[_MAX_PATH-1] = 0;
 
 	if( !CreateProcess( NULL, szPathOrig, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi ) ) {
 		common->Error( "Could not start process: '%s' ", szPathOrig );
